function [UBT] = f_read_UBT_2C(PATH, varargin)
% Reads UberTone-data after 1st processing step. Input is a string or
% character referring to the directory where UberTone-data is stored.
% 
% Output variables are:
% 
% UBT: Structure with the following fields:
% 
% Unprocessed data fields:
% - VAv: averaged (by UB-Lab itself) beam velocities as in the file "velocity_[meas_date]_[timestamp].csv"
% - VInst: instantaneous beam velocities as in the file "instvelocity_[meas_date]_[timestamp].csv"
% - AAv: averaged (by UB-Lab itself) echo amplitude (v) as in the file "amplitude_[meas_date]_[timestamp].csv"
% - AInst: instanteneous echo amplitude (V) as in the file "instamplitude_[meas_date]_[timestamp].csv"
% - itimes: timestamps (matlab format) corresponding the the instantaneous amplitude and -velocity data
% 
% Processed data fields:
% - V1_proc, V2_proc (Tav x X): averaged (by UB-Lab itself) beam velocities corresponding to beam 1 resp. beam 2
% - V1, V2 (T x X): instantaneous beam velocities corresponding to beam 1 resp. beam 2
% - A1_proc, A2_proc (Tav x X): averaged (by UB-Lab itself) echo amplitude (v) corresponding to beam 1 resp. beam 2
% - A1, A2 (T x X): instanteneous echo amplitude (V) corresponding to beam 1 resp. beam 2
% - itimes (T x 1): timestamps (matlab format) corresponding the the instantaneous amplitude and -velocity data
% - ptimes (Tav x 1): timestamps (matlab format) corresponding the the averaged amplitude and -velocity data
% - pdist (1 x X): distance to transducer
% - Tr: transducer ID's
% - Tr1idx, Tr2idx (1 x 2X): indicating data corresponding to transducer 1 resp. 2.  Allows to reference all data in Vinst and Ainst corresponding to Tr 1 or 2.
% 
% Derived data fields:
% - alpha (1 x X): doppler angle alpha. Calculated with gamma=2.09rad and d0=8cm.
% - U (T x X): instantaneous velocity in stream direction.
% - W (T x X): instatnaneous vertical velocity
% U and W are derived from the instantaneous beam velocities (VInst) if
% present. If these are NOT present in the data directory, U and W will be
% derived from the averaged beam velocities (VAv). A warning will be given.
% 
% bed_pos: estimated bed position (distance from the transducer) based on echo amplitude of the first transducer only!
% (To do: bed_pos for all transducers)
% 
% Last edit:  '14-Sep-2022 12:26:46'

%% Parse input
P = inputParser;
defaultCSVexport = 0;

validstring = @(x) isstring(x) | ischar(x);
% validbool = @(x) (boolean(x));
addRequired(P, 'PATH', validstring);
addOptional(P, 'write_to_csv', defaultCSVexport);

parse(P, PATH, varargin{:});
write_to_csv = P.Results.write_to_csv;

%% Start processing

% From settings file:
gamma = 2.09;       % rad
d0 = 0.08;          % m

config = [1];

wd = pwd;

% if SubFolders
%     files_uinst = dir([PATH, '\*\instvelocity*.csv']);
%     files_Ainst = dir([PATH, '\*\instamplitude*.csv']);
% 
%     files_uproc = dir([PATH, '\*\velocity*.csv']);
%     files_snr = dir([PATH, '\*\snr*.csv']);
%     files_Aproc = dir([PATH, '\*\amplitude*.csv']);
%     files_sets = dir([PATH, '\*\settings*.xml']);
% end

if ~ischar(PATH)
    if ~isstring(PATH)
    error('PATH must be provided as a string.')
    end
end

cd(PATH)        % Switch to datafolder
disp(['Data-folder found. Start reading data from ', PATH])

filename_uinst = dir('instvelocity*.csv');
filename_uproc = dir('velocity*.csv');
filename_snr = dir('snr*.csv');
filename_Ainst = dir('instamplitude*.csv');
filename_Aproc = dir('amplitude*.csv');
filename_sets = dir('settings*.xml');

instdata = 1;
if isempty(filename_uinst)
    instdata = 0;
    warning('No instantaneous measurements found. Reading averaged data only.\n%s',...
        'U and W will be calculated based on averaged velocity data.')
end

%% Read data

for c = 1:length(config)
    OPTS = detectImportOptions(filename_uproc(c).name);  OPTS = setvaropts(OPTS, "Var1", 'InputFormat','dd/MM/uuuu');
    UBT(c).VAv = readtable(filename_uproc(c).name, OPTS);
    
    if instdata
    OPTS = detectImportOptions(filename_uinst(c).name, 'Delimiter', '\t');  OPTS = setvaropts(OPTS, "Var1", 'InputFormat','dd/MM/uuuu');
    UBT(c).VInst = readtable(filename_uinst(c).name, OPTS);
    end
    
    OPTS = detectImportOptions(filename_Aproc(c).name);  OPTS = setvaropts(OPTS, "Var1", 'InputFormat','dd/MM/uuuu');
    UBT(c).AAv = readtable(filename_Aproc(c).name, OPTS);
    
    if instdata
    OPTS = detectImportOptions(filename_Ainst(c).name, 'Delimiter', '\t');  OPTS = setvaropts(OPTS, "Var1", 'InputFormat','dd/MM/uuuu');
    UBT(c).AInst = readtable(filename_Ainst(c).name, OPTS);
    end
    
    UBT(c).Tr = [UBT(c).VAv.Var3(2), UBT(c).VAv.Var4(2)];   % Transducer ID's
    
        % Remove empty data columns (why is this needed for certain datafiles?)
    if instdata
        keepcol = zeros([width(UBT(c).VInst), 1]);
        for cols = 1:width(UBT(c).VInst)
            if ~iscell(UBT(c).VInst{:,cols})
                keepcol(cols) = 1;
            end
        end
        UBT(c).VInst = UBT(c).VInst(:, find(keepcol));
        UBT(c).AInst = UBT(c).AInst(:, find(keepcol));
    end
end

cd(wd)

%% Process velocity and echo data
% Process time information
for c = 1:length(config)
if instdata; UBT(c).itimes = datenum(UBT(c).VInst.Var1) + datenum(UBT(c).VInst.Var2);
UBT(c).itimes = UBT(c).itimes(4:end); end
UBT(c).ptimes = datenum(UBT(c).VAv.Var1) + datenum(UBT(c).VAv.Var2);
UBT(c).ptimes = UBT(c).ptimes(3:end);
end

for c = 1:length(config)
% Now remove first 2 columns from tables (datetime information)
UBT(c).VAv = UBT(c).VAv(:,3:end);
if instdata; UBT(c).VInst = UBT(c).VInst(:,3:end); end
UBT(c).AAv = UBT(c).AAv(:,3:end);
if instdata; UBT(c).AInst = UBT(c).AInst(:,3:end); end
% Convert to matrix
UBT(c).VAv = table2array(UBT(c).VAv);
if instdata; UBT(c).VInst = table2array(UBT(c).VInst); end
UBT(c).AAv = table2array(UBT(c).AAv);
if instdata; UBT(c).AInst = table2array(UBT(c).AInst); end
end

% Process velocity data
for c = 1:length(config)
% Calculate vertical distance from transducer
tmp_pdist = unique(UBT(c).VAv(1,:));    % Travelling distance from emitter
UBT(c).pdist = (d0^2 - tmp_pdist.^2)./(2*(d0*cos(gamma) - tmp_pdist));

% Find separate transducer indices
UBT(c).Tr1idx = UBT(c).VAv(2,:) == UBT(c).Tr(1);
UBT(c).Tr2idx = UBT(c).VAv(2,:) == UBT(c).Tr(2);

% Write transducer velocities to new matrix (V1 and V2)
if instdata; UBT(c).V1 = UBT(c).VInst(4:end, UBT(c).Tr1idx); end
if instdata; UBT(c).V2 = UBT(c).VInst(4:end, UBT(c).Tr2idx); end
UBT(c).V1_proc = UBT(c).VAv(3:end, UBT(c).Tr1idx);
UBT(c).V2_proc = UBT(c).VAv(3:end, UBT(c).Tr2idx);
% Write echo intensities to new matrix (A1 and A2)
if instdata; UBT(c).A1 = UBT(c).AInst(4:end, UBT(c).Tr1idx); end
if instdata; UBT(c).A2 = UBT(c).AInst(4:end, UBT(c).Tr2idx); end
UBT(c).A1_proc = UBT(c).AAv(3:end, UBT(c).Tr1idx);
UBT(c).A2_proc = UBT(c).AAv(3:end, UBT(c).Tr2idx);
end

%% Find bed position in amplitude data
% For now, based on transducer 1 only
[~, acmax] = max(UBT.A1_proc,[],2);
UBT.bed_pos = -UBT.pdist(acmax);

%% Compute u and w

for c = 1:length(config)
    % Calculate alpha based on geometry
    UBT(c).alpha = atan2(UBT(c).pdist - d0*cos(gamma), d0*sin(gamma)) - pi/2;
    alpha_deg = rad2deg(UBT(c).alpha);
    
    if instdata
        UBT.U = (UBT(c).V2 - UBT(c).V1)./(2*sind(alpha_deg));
        UBT.W = (UBT(c).V1 + UBT(c).V2)./(2+2*cosd(alpha_deg));
    end
    UBT.U_proc = (UBT(c).V2_proc - UBT(c).V1_proc)./(2*sind(alpha_deg));
    UBT.W_proc = (UBT(c).V1_proc + UBT(c).V2_proc)./(2+2*cosd(alpha_deg));
end

%% Write to csv-file if requested
if write_to_csv
    if instdata
        export_times = UBT.itimes;
    else
        export_times = UBT.ptimes;
    end

    export_tabU = array2table([export_times, UBT.U]);
    export_tabU.Var1 = datetime(export_tabU.Var1,'ConvertFrom','datenum');
    varnames = ['time', string(UBT.pdist)];
    export_tabU.Properties.VariableNames = varnames;
    writetable(export_tabU, [PATH, '\instvelocity_U.csv'], 'Delimiter', '\t');

    export_tabW = array2table([export_times, UBT.U]);
    export_tabW.Var1 = datetime(export_tabW.Var1,'ConvertFrom','datenum');
    varnames = ['time', string(UBT.pdist)];
    export_tabW.Properties.VariableNames = varnames;
    writetable(export_tabW, [PATH, '\instvelocity_W.csv'], 'Delimiter', '\t');
end

%% To do:
% Read d0 and gamma from corresponding settings file
% Find bed position for all transducers separately (differs per transducer
    % in case of a tilted UBT)

disp('Done!')

end