function UBT = f_read_UBT_4C_incl_rotation(PATH)
% Reads UberTone-data after 1st processing step. Input is a string or
% character referring to the directory where UberTone-data is stored.
% 
% Output variables are:
% 
% UBT: Structure with the following fields:
% 
% Unprocessed data fields:
% - VAv: averaged (by UB-Lab itself) beam velocities as in the file "velocity_[meas_date]_[timestamp].csv"
% - VInst: instantaneous beam velocities as in the file "instvelocity_[meas_date]_[timestamp].csv"
% - AAv: averaged (by UB-Lab itself) echo amplitude (v) as in the file "amplitude_[meas_date]_[timestamp].csv"
% - AInst: instanteneous echo amplitude (V) as in the file "instamplitude_[meas_date]_[timestamp].csv"
% - itimes: timestamps (matlab format) corresponding the the instantaneous amplitude and -velocity data
% 
% Processed data fields:
% - V1_proc, V2_proc (Tav x X): averaged (by UB-Lab itself) beam velocities corresponding to beam 1 resp. beam 2
% - V1, V2 (T x X): instantaneous beam velocities corresponding to beam 1 resp. beam 2
% - A1_proc, A2_proc (Tav x X): averaged (by UB-Lab itself) echo amplitude (v) corresponding to beam 1 resp. beam 2
% - A1, A2 (T x X): instanteneous echo amplitude (V) corresponding to beam 1 resp. beam 2
% - itimes (T x 1): timestamps (matlab format) corresponding the the instantaneous amplitude and -velocity data
% - ptimes (Tav x 1): timestamps (matlab format) corresponding the the averaged amplitude and -velocity data
% - pdist (1 x X): distance to transducer
% - Tr: transducer ID's
% - Tr1idx, Tr2idx (1 x 2X): indicating data corresponding to transducer 1/3 resp. 2/4.  Allows to reference all data in Vinst and Ainst corresponding to Tr 1/3 or 2/4.
% 
% Derived data fields 
% - U: streamwise velocity (assuming theta = 45 deg) - Corresponding timestamps are UBT.ptimes
% - V: normal velocity (assuming theta = 45 deg) - Corresponding timestamps are UBT.ptimes
% - W: vertical velocity (assuming z_instr = z_earth) - Corresponding timestamps are UBT.ptimes
% U and W are derived from the instantaneous beam velocities (VInst) if
% present. If these are NOT present in the data directory, U and W will be
% derived from the averaged beam velocities (VAv). A warning will be given.
% 
% bed_pos: estimated bed position (distance from the transducer) based on echo amplitude of the first transducer only!
% (To do: bed_pos for all transducers)
% 
% Last edit: '14-Sep-2022 13:52:00'
 
wd = pwd;
config = [1,2];

% To do: read these from settings file
gamma = 2.09;       % rad
d0 = 0.08;          % m
theta = pi/4;       % rad

cd(PATH)        % Switch to datafolder

filename_uinst = dir('instvelocity*.csv');
filename_uproc = dir('velocity*.csv');
filename_snr = dir('snr*.csv');
filename_Ainst = dir('instamplitude*.csv');
filename_Aproc = dir('amplitude*.csv');
filename_sets = dir('settings*.xml');

instdata = 1;
if isempty(filename_uinst)
    instdata = 0;
    warning('No instantaneous measurements found. Reading averaged data only.\n%s',...
        'U and W will be calculated based on averaged velocity data.')
end

%% Read data
Tr = [1,2;3,4];

for c = 1:length(config)
    OPTS = detectImportOptions(filename_uproc(c).name);  OPTS = setvaropts(OPTS, "Var1", 'InputFormat','dd/MM/uuuu');
    UBT(c).VAv = readtable(filename_uproc(c).name, OPTS);
    
    if instdata
    OPTS = detectImportOptions(filename_uinst(c).name, 'Delimiter', '\t');  OPTS = setvaropts(OPTS, "Var1", 'InputFormat','dd/MM/uuuu');
    UBT(c).VInst = readtable(filename_uinst(c).name, OPTS);
    end
    
    OPTS = detectImportOptions(filename_Aproc(c).name);  OPTS = setvaropts(OPTS, "Var1", 'InputFormat','dd/MM/uuuu');
    UBT(c).AAv = readtable(filename_Aproc(c).name, OPTS);
    
    if instdata
    OPTS = detectImportOptions(filename_Ainst(c).name, 'Delimiter', '\t');  OPTS = setvaropts(OPTS, "Var1", 'InputFormat','dd/MM/uuuu');
    UBT(c).AInst = readtable(filename_Ainst(c).name, OPTS);
    end
    
    UBT(c).Tr = Tr(c,:);
end

cd(wd)         % Restore old working directory
%% Process velocity and echo data
% Process time information
for c = 1:length(config)
    if instdata
    UBT(c).itimes = datenum(UBT(c).VInst.Var1) + datenum(UBT(c).VInst.Var2);
    UBT(c).itimes = UBT(c).itimes(4:end);
    end
    UBT(c).ptimes = datenum(UBT(c).VAv.Var1) + datenum(UBT(c).VAv.Var2);
    UBT(c).ptimes = UBT(c).ptimes(3:end);
end

% I don't like that this is necessary: sometimes, the UBT skips a
% measurement with either pair 1 or pair 2... Here, make sure that we have
% an equal number of blocks with both configurations.
if length(UBT(1).ptimes) ~= length(UBT(2).ptimes)
    % Find configuration with excess of measurements:
    conf_exc = find(max(length(UBT(1).ptimes), length(UBT(2).ptimes)) == max(length(UBT(1).ptimes), length(UBT(2).ptimes)));

    [~,tt] = sort([UBT(1).ptimes; UBT(2).ptimes]);
    tt = diff(tt);
    idx_bad = ceil(find(tt == 1)/2); % This is where 2 measurements with the same pair of recievers follow up on each other

    idx_good = 1:length(UBT(conf_exc).ptimes);
    for n = 1:length(idx_bad)
        idx_good = [idx_good(1:idx_bad(n)-1), idx_good(idx_bad(n)+1:end)];
    end
    clear tt idx_bad
end

for c = 1:length(config)
    % Now remove first 2 columns from tables (datetime information)
    UBT(c).VAv = UBT(c).VAv(:,3:end);
    if instdata; UBT(c).VInst = UBT(c).VInst(:,3:end); end
    UBT(c).AAv = UBT(c).AAv(:,3:end);
    if instdata; UBT(c).AInst = UBT(c).AInst(:,3:end); end
    % Convert to matrix
    UBT(c).VAv = table2array(UBT(c).VAv);
    if instdata; UBT(c).VInst = table2array(UBT(c).VInst); end
    UBT(c).AAv = table2array(UBT(c).AAv);
    if instdata; UBT(c).AInst = table2array(UBT(c).AInst); end
end

% Remove excess data from all processed/averaged variables (found in line 102 - 115)
if length(UBT(1).ptimes) ~= length(UBT(2).ptimes)
    UBT(conf_exc).ptimes = UBT(conf_exc).ptimes(idx_good);
    UBT(conf_exc).VAv = [UBT(conf_exc).VAv(1:2,:); UBT(conf_exc).VAv(idx_good+2,:)];
    UBT(conf_exc).AAv = [UBT(conf_exc).AAv(1:2,:); UBT(conf_exc).AAv(idx_good+2,:)];
end

% Process velocity data
for c = 1:length(config)
    % Calculate vertical distance from transducer
    tmp_pdist = unique(UBT(c).VAv(1,:));    % Travelling distance from emitter
    UBT(c).pdist = (d0^2 - tmp_pdist.^2)./(2*(d0*cos(gamma) - tmp_pdist));

    % Find separate transducer indices
    UBT(c).Tr1idx = UBT(c).VAv(2,:) == UBT(c).Tr(1);
    UBT(c).Tr2idx = UBT(c).VAv(2,:) == UBT(c).Tr(2);
    
    % Write transducer velocities to new matrix (V1 and V2)
    if instdata; UBT(c).V1 = UBT(c).VInst(3:end, UBT(c).Tr1idx);
    UBT(c).V2 = UBT(c).VInst(3:end, UBT(c).Tr2idx); end
    UBT(c).V1_proc = UBT(c).VAv(3:end, UBT(c).Tr1idx);
    UBT(c).V2_proc = UBT(c).VAv(3:end, UBT(c).Tr2idx);
    % Write echo intensities to new matrix (A1 and A2)
    if instdata; UBT(c).A1 = UBT(c).AInst(3:end, UBT(c).Tr1idx);
    UBT(c).A2 = UBT(c).AInst(3:end, UBT(c).Tr2idx); end
    UBT(c).A1_proc = UBT(c).AAv(3:end, UBT(c).Tr1idx);
    UBT(c).A2_proc = UBT(c).AAv(3:end, UBT(c).Tr2idx);
end

%% Find bed position in amplitude data
% For now, based on transducer 1 only
for c = 1:length(config)
[~, acmax] = max(UBT(c).A1_proc,[],2);
UBT(c).bed_pos = -UBT(c).pdist(acmax);
end

%% Conversion to instrument velocities

% Step 1: calculate velocities in instrument-coordinates
for c = 1:length(config)
% Calculate alpha based on geometry
UBT(c).alpha = atan2(UBT(c).pdist - d0*cos(gamma), d0*sin(gamma)) - pi/2;
alpha_deg = rad2deg(UBT(c).alpha);

% if instdata
%     UBT(c).U_instr = (UBT(c).V2 - UBT(c).V1)./(2*sind(alpha_deg));      % x-velocity in instruments coordinate system
%     UBT(c).W = (UBT(c).V1 + UBT(c).V2)./(2+2*cosd(alpha_deg));      % z-velocity in instruments coordinate system (usually equal to z in earth coordinates)
% else
    UBT(c).U_instr = (UBT(c).V2_proc - UBT(c).V1_proc)./(2*sind(alpha_deg));      % x-velocity in instruments coordinate system
    UBT(c).W = (UBT(c).V1_proc + UBT(c).V2_proc)./(2+2*cosd(alpha_deg));      % z-velocity in instruments coordinate system (usually equal to z in earth coordinates)
% end
end

%% Rotation to streamwise components

% Step 2: Rotation to earth coordinates

U_instr_1 = UBT(1).U_instr;
U_instr_2 = UBT(2).U_instr;

% Perform actual rotation
rmat = [cos(theta), -sin(theta);...
    sin(theta), cos(theta)];

U_s = zeros(size(U_instr_1));
U_n = U_s;

for n = 1:numel(U_s)
    U = rmat*([U_instr_1(n); U_instr_2(n)]);        % yields vector (U_s; U_n)
    U_s(n) = U(1);
    U_n(n) = U(2);
end

% Write to output structure
for c = 1:length(config)
    UBT(c).U = U_s;
    UBT(c).V = U_n;
end

%% To do:
% Read d0 and gamma from corresponding settings file
% Find bed position for all transducers separately (differs per transducer
    % in case of a tilted UBT)

end